/* FILE: avftoave.cc                 -*-Mode: c++-*-
 *
 * Program to convert from Any known, regular Vector Field
 * file to an data file of magnetization averages along the
 * x, y and z axes.
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "oc.h"
#include "nb.h"
#include "vf.h"

/* End includes */

static char *inname;
static Vf_Mesh *mesh;

void Usage(void)
{
  fprintf(stderr,"Usage: avftoave infile >outfile\n");
  exit(1);
}

extern "C" {
static int
MyTcl_AppInit(Tcl_Interp *interp)
{
    Oc_AutoBuf ab;
    if (Oc_Init(interp) == TCL_ERROR) {
        return TCL_ERROR;
    }
    if (Vf_Init(interp) == TCL_ERROR) {
        return TCL_ERROR;
    }

    // Fill mesh from input file
    Vf_FileInput *vffreader = Vf_FileInput::NewReader(inname);
    if(vffreader==NULL) { // Unknown file type or other error
        fprintf(stderr,"ERROR detected processing file %s\n",inname);
        return Tcl_GlobalEval(interp, ab("exit 1"));
    }
    else { // Reader found for inname
        mesh=vffreader->NewMesh();
        delete vffreader;
        if(mesh==NULL) {
            fprintf(stderr,"ERROR detected while reading file %s\n",inname);
            return Tcl_GlobalEval(interp, ab("exit 2"));
        }
    }

    // Is this a rectangular mesh?
    if(strcmp(mesh->GetMeshType(),"Vf_GridVec3f")!=0) {
      fprintf(stderr,"Invalid mesh type: %s\n",mesh->GetMeshType());
      return Tcl_GlobalEval(interp, ab("exit 2"));
    }

    // Downcast
    Vf_GridVec3f& grid = *((Vf_GridVec3f *)mesh);

    fprintf(stdout,"# Data averages for file %s\n#\n",inname);

    // Collect and print x-data
    int i,j,k,isize,jsize,ksize;
    grid.GetDimens(isize,jsize,ksize);
    if(isize<1 || jsize<1 || ksize<1) {
      fprintf(stderr,"Bad mesh dimensions: %d %d %d",
              isize,jsize,ksize);
      return Tcl_GlobalEval(interp, ab("exit 2"));
    }
    Nb_Vec3<REAL4> *m=new Nb_Vec3<REAL4>[isize];
    for(i=0;i<isize;i++) m[i].Set(0.,0.,0.);
    for(i=0;i<isize;i++) for(j=0;j<jsize;j++) for(k=0;k<ksize;k++) {
      m[i]+=grid(i,j,k);
    }
    Nb_Vec3<REAL4> base=grid.GetBasePoint();
    Nb_Vec3<REAL4> step=grid.GetGridStep();

    double mult=(1./double(jsize*ksize));
    fprintf(stdout,"# Averages across x axis (%d points each)\n",
            jsize*ksize);
    for(i=0;i<isize;i++) {
      m[i]*=mult;
      fprintf(stdout,"%- #12.6g    %- #12.6g  %- #12.6g  %- #12.6g\n",
              base.x+double(i)*step.x,m[i].x,m[i].y,m[i].z);
    }
    fflush(stdout);
    delete[] m;

    // Collect and print y-data
    m=new Nb_Vec3<REAL4>[jsize];
    for(j=0;j<jsize;j++) m[j].Set(0.,0.,0.);
    for(i=0;i<isize;i++) for(j=0;j<jsize;j++) for(k=0;k<ksize;k++) {
      m[j]+=grid(i,j,k);
    }
    mult=(1./double(isize*ksize));
    fprintf(stdout,"\n\n# Averages across y axis (%d points each)\n",
            isize*ksize);
    for(j=0;j<jsize;j++) {
      m[j]*=mult;
      fprintf(stdout,"%- #12.6g    %- #12.6g  %- #12.6g  %- #12.6g\n",
              base.y+double(j)*step.y,m[j].x,m[j].y,m[j].z);
    }
    fflush(stdout);
    delete[] m;

    // Collect and print z-data
    m=new Nb_Vec3<REAL4>[isize];
    for(k=0;k<ksize;k++) m[k].Set(0.,0.,0.);
    for(i=0;i<isize;i++) for(j=0;j<jsize;j++) for(k=0;k<ksize;k++) {
      m[k]+=grid(i,j,k);
    }
    mult=(1./double(isize*jsize));
    fprintf(stdout,"\n\n#Averages across z axis (%d points each)\n",
            isize*jsize);
    for(k=0;k<ksize;k++) {
      m[k]*=mult;
      fprintf(stdout,"%- #12.6g    %- #12.6g  %- #12.6g  %- #12.6g\n",
              base.z+double(k)*step.z,m[k].x,m[k].y,m[k].z);
    }
    fflush(stdout);
    delete[] m;

    // That's all folks!
    delete mesh;
    fflush(stdout);
    return Tcl_GlobalEval(interp, ab("exit 0"));
}
}

int Oc_Main(int argc,char **argv)
{
  if(argc!=2) Usage();
  inname=argv[1];
  Tcl_Main(argc, argv, MyTcl_AppInit);
  return 0;
}

